==> atmsupport-1.0.tar.gz: atmsupport-1.0/libats/ats.h <==
/*
 * Copyright (c) 2003-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Begemot: atmsupport/libats/ats.h,v 1.10 2003/09/25 13:47:59 hbb Exp $
 */
#include <sys/types.h>
#include <sys/queue.h>
#include <isc/eventlib.h>

struct func;

/*
 * This is a position in an input stream
 */
struct ipos {
	char	*name;		/* function or file name */
	u_int	lno;		/* line number */
};

/*
 * Structure to store a list of strings
 */
struct string {
	TAILQ_ENTRY(string) link;
	u_int		lno;
	char		*string;
};
TAILQ_HEAD(string_list, string);

/*
 * Structure to hold a line while beeing worked on
 */
struct line {
	char		*line;	/* start of buffer */
	char		*ptr;	/* current pointer */
	size_t		alloc;	/* allocated size */
};

/*
 * This is an input stream
 */
struct input {
	struct ipos	pos;	/* current position */
	u_int		type;	/* type of this input */
	struct input	*stack;	/* calling input */

	union {
	    struct {
		FILE	*fp;	/* if !NULL really a file */
		size_t	len;	/* size of line in buffer */
	    }		file;
	    struct {
		struct func *func;	/* function */
		struct string *curr;	/* current function pointer */
	    }		func;
	    struct {
		struct string_list block;	/* block to execute */
		struct string *cond;		/* condition to test */
		struct string *curr;		/* current line */
	    }		loop;
	}		u;
};
#define	INPUT_FILE	0
#define	INPUT_FUNC	1
#define	INPUT_LOOP	2

#define	file_fp		u.file.fp
#define	file_len	u.file.len
#define	func_func	u.func.func
#define	func_curr	u.func.curr
#define	loop_block	u.loop.block
#define	loop_cond	u.loop.cond
#define	loop_curr	u.loop.curr

/*
 * A function
 */
struct func {
	const char	*name;		/* name of the function */
	struct ipos	def;		/* where it has been defined */
	u_int		nargs;		/* number of formal args */
	SLIST_ENTRY(func) link;		/* all functions list */
	int		repl;		/* string replacement? */

	struct string_list lines;	/* lines in this function */
	struct string_list args;	/* formal argument names */
	const char	*str;		/* replacement string */
};

/*
 * This structure stores a variable value that can be pushed when
 * a new namespace is opened.
 */
struct value {
	struct value	*stack;		/* pushed definitions */
	char		*val;		/* value of the variable */
	u_int		def_level;	/* definition level */
	struct value	*ref;		/* if this is a reference */
	char		*(*get_val)(struct value *);
	void		(*set_val)(struct value *, const char *, size_t);
};

/*
 * This is a variable
 */
struct var {
	char		*name;		/* name of the variable */
	struct value	*v;		/* value stack */
	LIST_ENTRY(var)	link;		/* links all variables */
};
LIST_HEAD(var_list, var);

/*
 * This structure describes a key-action mapping
 */
struct key {
	const char	*key;		/* action key */
	int		run;		/* when is this key legal */
	int		subst : 1;	/* do substitution? */
	int		skip : 1;	/* skip when skipped */
	void		(*func)(struct line *);	/* the executor */
};

#define	ACT_EITHER	0
#define	ACT_RUNNING	1
#define	ACT_STOPPED	2

/*
 * A list of strings to wait for
 */
struct waitlist {
	const char	*name;		/* name of this list */
	struct string_list wait;	/* what to wait for */
	LIST_ENTRY(waitlist) link;
};

/*
 * A list of strings to hold until next event loop
 */
struct holdlist {
	const char	*name;		/* name of this list */
	struct string_list hold;	/* what to wait for */
	struct waitlist	*waitlist;
	LIST_ENTRY(holdlist) link;
};

/*
 * A list of strings to ignore
 */
struct ignlist {
	const char	*name;		/* name of this list */
	struct string_list ignore;	/* what to ignore */
	u_int		gen;
};

/* the event context */
extern evContext evctx;

/* current test name */
extern char *current_test_name;

/* skip this test */
extern int skip_test;

/* function call stack */
extern struct input *input;

/* current definition level */
extern u_int var_level;

/* additional user keys */
extern const struct key usrkeys[];
extern const struct key usrgrpkeys[];

/* current trace level */
extern u_int trace_level;

/* reporting level */
extern u_int report_level;

/* Create a new line */
struct line *line_create(size_t);

/* Destroy a line */
void line_destroy(struct line *);

/* Append string to line */
void line_append_string(struct line *, const char *);

/* Append char to line */
void line_append_char(struct line *, int);

/* problem report */
void report_problem(const char *, ...) __printflike(1, 2);

/* Init a test */
void report_test_init(void);

/* Report at end of test */
void report_test_result(void);

/* Report total results */
u_int report_total(void);

/* Write a trace message */
void trace(u_int, const char *, ...);

/* report error and exit */
void err_file(const char *fmt, ...) __dead2 __printflike(1, 2);

/* report a warning */
void warn_file(const char *fmt, ...) __printflike(1, 2);

/* return the variable who's name starts at the given pointer */
struct var *var_find(const char *, const char **);

/* Set the variable's value to the given string. */
void var_setn(struct var *, const char *, size_t);

/* create or set a variable */
struct var *var_define(const char *, const char *, int, struct var *);

/* create or set a global variable */
struct var *var_define_global(const char *, const char *);

/* Pop all variables that have been defined above this level. */
void var_pop(u_int);

/* Make substitution on the given variable */
const char *var_subst(const char *, const char **);

/* Return a variable's value */
const char *var_getval(const struct var *var);

/* create special variables */
void var_create_special(void);

/* Make expression substitution on the given expression */
char *expr_subst(const char *, const char **);

/* Open a file for input */
struct input *input_open_file(const char *);

/* Open a function for input */
struct input *input_open_func(struct func *);

/* Open a loop and push into onto the input stack */
struct input *input_open_loop(struct string *, struct string_list *);

/* Read the next line */
int read_line(struct line *);

/* Skip blanks */
char *skip_blanks(struct line *);

/* Skip to the end of the current input line */
void skip_line(struct line *);

/* Find the begin and the length of the next word */
char *peek_word(struct line *, size_t *);

/* Eat the last peeked word (and nul-terminate it) */
void swallow_word(struct line *, size_t);

/* Get the next word (nul-terminated). */
char *get_word(struct line *);

/* Get a variable name */
char *get_name(struct line *);

/* Replace all space sequences in the given line by exactly one space. */
void compress_blanks(char *);

/* Save the given string */
struct string *string_save(const char *, u_int);
struct string *string_saven(const char *, u_int, size_t);

/* Free a string list */
void stringlist_free(struct string_list *);

/* Do ! and $ substitution on a line. */
void substitute_line(struct line *);

/* Dump all functions */
void funcs_dump(void);

/* Define a function. */
struct func *func_define(const char *, const struct ipos *,
    struct string_list *, struct string_list *);

/* Call the named function */
void func_call(const char *, const struct string_list *);

/* Define a replacement */
void replace_define(const char *, const struct ipos *, struct string_list *,
    const char *);

/* Make substitution on the given string */
struct line *str_subst(const char *, const char **);

/* Replace part of the line by another string */
void splice(struct line *, size_t, const char *);

/* Read and execute lines until the bottom of the stack is reached. */
void test_exec(void);

/* Callback: Return non-zero if we are waiting for something */
int test_wait_cb(void);

/* Wait for events to happen */
void test_wait_events(void);

/* Initialize test stuff */
void test_init(void);

/* Callback to start a test */
void test_start_cb(void);

/* Callback to reset a test */
void test_reset_cb(int tout);

/* create a waitlist */
struct waitlist *waitlist_create(const char *);

/* Put the rest of the current line on a wait list */
void waitlist_put(struct waitlist *, struct line *);

/* create a holdlist */
struct holdlist *holdlist_create(const char *, struct waitlist *);

/* Put the rest of the current line on a hold list */
void holdlist_put(struct holdlist *, struct line *);

/* Try to match the given string with a waitlist */
int waitlist_match(struct waitlist *, const char *);
int waitlist_match_ign(struct waitlist *, const char *);

/* create a ignlist */
struct ignlist *ignlist_create(const char *);

/* Put the rest of the current line on a ign list */
void ignlist_push(struct ignlist *, int, struct line *);

/* Try to match the given string with a ignlist */
int ignlist_match(struct ignlist *, const char *);

/* pop ignore */
void ignlist_pop(struct ignlist *);

/* Print input stack */
void print_stack(const struct input *);

/* check/set verbosity flag */
extern uint8_t verb_flag[256 / 8];

#define	ISVERB(F) \
		(verb_flag[((F) & 0xff) / 8] & (1 << (((F) & 0xff) % 8)))

#define	SETVERB(F) do { \
	    verb_flag[((F) & 0xff) / 8] |= (1 << (((F) & 0xff) % 8)); \
    } while (0)

==> atmsupport-1.3.tar.gz: atmsupport-1.3/libats/ats.h <==
/*
 * Copyright (c) 2003-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Begemot: atmsupport/libats/ats.h,v 1.15 2004/07/08 13:33:34 brandt Exp $
 */

#ifndef _ATS_H_
#define _ATS_H_

#include <sys/types.h>
#include <sys/queue.h>
#include <stdio.h>
#include <stdint.h>
#ifndef USE_BEGEMOT
#include <isc/eventlib.h>
#endif

struct func;

/*
 * This is a position in an input stream
 */
struct ipos {
	char	*name;		/* function or file name */
	u_int	lno;		/* line number */
};

/*
 * Structure to store a list of strings
 */
struct string {
	TAILQ_ENTRY(string) link;
	u_int		lno;
	char		*string;
};
TAILQ_HEAD(string_list, string);

/*
 * Structure to hold a line while beeing worked on
 */
struct line {
	char		*line;	/* start of buffer */
	char		*ptr;	/* current pointer */
	size_t		alloc;	/* allocated size */
};

/*
 * This is an input stream
 */
struct input {
	struct ipos	pos;	/* current position */
	u_int		type;	/* type of this input */
	struct input	*stack;	/* calling input */

	union {
	    struct {
		FILE	*fp;	/* if !NULL really a file */
		size_t	len;	/* size of line in buffer */
	    }		file;
	    struct {
		struct func *func;	/* function */
		struct string *curr;	/* current function pointer */
	    }		func;
	    struct {
		struct string_list block;	/* block to execute */
		struct string *cond;		/* condition to test */
		struct string *curr;		/* current line */
	    }		loop;
	}		u;
};
#define	INPUT_FILE	0
#define	INPUT_FUNC	1
#define	INPUT_LOOP	2

#define	file_fp		u.file.fp
#define	file_len	u.file.len
#define	func_func	u.func.func
#define	func_curr	u.func.curr
#define	loop_block	u.loop.block
#define	loop_cond	u.loop.cond
#define	loop_curr	u.loop.curr

/*
 * A function
 */
struct func {
	const char	*name;		/* name of the function */
	struct ipos	def;		/* where it has been defined */
	u_int		nargs;		/* number of formal args */
	LIST_ENTRY(func) link;		/* all functions list */
	int		repl : 1;	/* string replacement? */
	int		stat : 1;	/* file static */
	u_int		file_level;

	struct string_list lines;	/* lines in this function */
	struct string_list args;	/* formal argument names */
	const char	*str;		/* replacement string */
};

/*
 * This structure stores a variable value that can be pushed when
 * a new namespace is opened.
 */
struct value {
	struct value	*stack;		/* pushed definitions */
	char		*val;		/* value of the variable */
	u_int		def_level;	/* definition level */
	struct value	*ref;		/* if this is a reference */
	char		*(*get_val)(struct value *);
	void		(*set_val)(struct value *, const char *, size_t);
	char		*reset;		/* reset value */
};

/*
 * This is a variable
 */
struct var {
	char		*name;		/* name of the variable */
	struct value	*v;		/* value stack */
	LIST_ENTRY(var)	link;		/* links all variables */
};
LIST_HEAD(var_list, var);

/*
 * This structure describes a key-action mapping
 */
struct key {
	const char	*key;		/* action key */
	int		run;		/* when is this key legal */
	int		subst : 1;	/* do substitution? */
	int		skip : 1;	/* skip when skipped */
	void		(*func)(struct line *);	/* the executor */
};

#define	ACT_EITHER	0
#define	ACT_RUNNING	1
#define	ACT_STOPPED	2

/*
 * A list of strings to wait for
 */
struct waitlist {
	const char	*name;		/* name of this list */
	struct string_list wait;	/* what to wait for */
	LIST_ENTRY(waitlist) link;
};

/*
 * A list of strings to hold until next event loop
 * The will be moved over to the waitlist on the next round.
 */
struct holdlist {
	const char	*name;		/* name of this list */
	struct string_list hold;	/* what to wait for */
	struct waitlist	*waitlist;
	LIST_ENTRY(holdlist) link;
};

/*
 * A list of strings to ignore
 */
struct ignlist {
	const char	*name;		/* name of this list */
	struct string_list ignore;	/* what to ignore */
	u_int		gen;
};

/*
 * A list of event strings to be checked in the next event round.
 * They will be checked against the given waitlist.
 */
struct eventlist {
	const char	*name;		/* name of this list */
	struct string_list events;	/* event strings */
	struct waitlist *waitlist;
	LIST_ENTRY(eventlist) link;
};

/* current test name */
extern char *current_test_name;

/* skip this test */
extern int skip_test;

/* function call stack */
extern struct input *input;

/* file level */
extern u_int input_file_level;

/* current definition level */
extern u_int var_level;

/* additional user keys */
extern const struct key usrkeys[];
extern const struct key usrgrpkeys[];

/* current trace level */
extern u_int trace_level;

/* reporting level */
extern u_int report_level;

/* current key */
extern const char *curr_key;

/* Create a new line */
struct line *line_create(size_t);

/* Destroy a line */
void line_destroy(struct line *);

/* Append string to line */
void line_append_string(struct line *, const char *);

/* Append char to line */
void line_append_char(struct line *, int);

/* problem report */
void report_problem(const char *, ...) __printflike(1, 2);

/* Init a test */
void report_test_init(void);

/* Report at end of test */
void report_test_result(void);

/* Report total results */
u_int report_total(void);

/* Write a trace message */
void trace(u_int, const char *, ...);

/* report error and exit */
void err_file(const char *fmt, ...) __dead2 __printflike(1, 2);

/* report a warning */
void warn_file(const char *fmt, ...) __printflike(1, 2);

/* return the variable who's name starts at the given pointer */
struct var *var_find(const char *, const char **);

/* Set the variable's value to the given string. */
void var_setn(struct var *, const char *, size_t);

/* create or set a variable */
struct var *var_define(const char *, const char *, int, struct var *);

/* create or set a global variable */
struct var *var_define_global(const char *, const char *);

/* create and set a resetable variable */
struct var *var_define_resetable(const char *, const char *);

/* pop all local variable */
void var_reset(void);

/* Pop all variables that have been defined above this level. */
void var_pop(u_int);

/* Make substitution on the given variable */
const char *var_subst(const char *, const char **);

/* Return a variable's value */
const char *var_getval(const struct var *var);

/* create special variables */
void var_create_special(void);

/* Make expression substitution on the given expression */
char *expr_subst(const char *, const char **);

/* Open a file for input */
struct input *input_open_file(const char *);

/* Open a function for input */
struct input *input_open_func(struct func *);

/* Open a loop and push into onto the input stack */
struct input *input_open_loop(struct string *, struct string_list *);

/* Read the next line */
int read_line(struct line *);

/* Skip blanks */
char *skip_blanks(struct line *);

/* Skip to the end of the current input line */
void skip_line(struct line *);

/* Find the begin and the length of the next word */
char *peek_word(struct line *, size_t *);

/* Eat the last peeked word (and nul-terminate it) */
void swallow_word(struct line *, size_t);

/* Get the next word (nul-terminated). */
char *get_word(struct line *);

/* Get the next possibly quoted word (nul-terminated). */
char *get_qword(struct line *);

/* Get a variable name */
char *get_name(struct line *);

/* Replace all space sequences in the given line by exactly one space. */
void compress_blanks(char *);

/* Save the given string */
struct string *string_save(const char *, u_int);
struct string *string_saven(const char *, u_int, size_t);

/* Free a string list */
void stringlist_free(struct string_list *);

/* Do ! and $ substitution on a line. */
void substitute_line(struct line *);

/* Dump all functions */
void funcs_dump(void);

/* Define a function. */
struct func *func_define(const char *, const struct ipos *,
    struct string_list *, struct string_list *, int);

/* Call the named function */
void func_call(const char *, const struct string_list *);

/* Destroy function */
void func_destroy(struct func *f);

/* Flush static functions */
void func_flush_static(void);

/* Define a replacement */
void replace_define(const char *, const struct ipos *, struct string_list *,
    const char *, int);

/* Make substitution on the given string */
struct line *str_subst(const char *, const char **);

/* Replace part of the line by another string */
void splice(struct line *, size_t, const char *);

/* Read and execute lines until the bottom of the stack is reached. */
void test_exec(void);

/* Callback: Return non-zero if we are waiting for something */
int test_wait_cb(void);

/* Wait for events to happen */
void test_wait_events(void);

/* Initialize test stuff */
void test_init(void);

/* Callback to start a test */
void test_start_cb(void);

/* Callback to reset a test */
void test_reset_cb(int tout);

/* create a waitlist */
struct waitlist *waitlist_create(const char *);

/* Put the rest of the current line on a wait list */
void waitlist_put(struct waitlist *, struct line *);

/* create a holdlist */
struct holdlist *holdlist_create(const char *, struct waitlist *);

/* Put the rest of the current line on a hold list */
void holdlist_put(struct holdlist *, struct line *);

/* create an eventlist */
struct eventlist *eventlist_create(const char *, struct waitlist *);

/* Put the string on the eventlist */
void eventlist_put(struct eventlist *, const char *);

/* Try to match the given string with a waitlist */
int waitlist_match(struct waitlist *, const char *);
int waitlist_match_ign(struct waitlist *, const char *);

/* create a ignlist */
struct ignlist *ignlist_create(const char *);

/* Put the rest of the current line on a ign list */
void ignlist_push(struct ignlist *, int, struct line *);

/* Try to match the given string with a ignlist */
int ignlist_match(struct ignlist *, const char *);

/* pop ignore */
void ignlist_pop(struct ignlist *);

/* match string and template */
int match(const char *templ, const char *s);

/* Print input stack */
void print_stack(const struct input *);

/* check/set verbosity flag */
extern uint8_t verb_flag[256 / 8];

#define	ISVERB(F) \
		(verb_flag[((F) & 0xff) / 8] & (1 << (((F) & 0xff) % 8)))

#define	SETVERB(F) do { \
	    verb_flag[((F) & 0xff) / 8] |= (1 << (((F) & 0xff) % 8)); \
    } while (0)

/* malloc with fatal exit */
void *ats_alloc(size_t);
void *ats_realloc(void *, size_t);

/* strdup with error check */
char *ats_strdup(const char *);

/* timers */
void *ats_start_timer(u_int, void (*)(void *), void *);
void ats_stop_timer(void *);

/* file, sockets */
void *ats_register_fd(int, void (*)(int, void *), void *);
void ats_unregister_fd(void *);

#endif
